From b2ea2c97eda39e4575da0f736553eecafd186ab8 Mon Sep 17 00:00:00 2001
From: Rushabh Lathia <rushabh.lathia@enterprisedb.com>
Date: Mon, 7 Apr 2025 13:51:22 +0530
Subject: [PATCH 3/3] Fix relation attribute loop correctly and also set the
 nullability to ATTNULLABLE_VALID for the catalog/temp tables.

---
 src/backend/access/common/tupdesc.c | 9 +++++++--
 src/backend/utils/cache/relcache.c  | 2 +-
 2 files changed, 8 insertions(+), 3 deletions(-)

diff --git a/src/backend/access/common/tupdesc.c b/src/backend/access/common/tupdesc.c
index 5831788cbff..d481fadaf73 100644
--- a/src/backend/access/common/tupdesc.c
+++ b/src/backend/access/common/tupdesc.c
@@ -22,6 +22,7 @@
 #include "access/htup_details.h"
 #include "access/toast_compression.h"
 #include "access/tupdesc_details.h"
+#include "catalog/catalog.h"
 #include "catalog/pg_collation.h"
 #include "catalog/pg_type.h"
 #include "common/hashfn.h"
@@ -74,8 +75,12 @@ populate_compact_attribute_internal(Form_pg_attribute src,
 	dst->atthasmissing = src->atthasmissing;
 	dst->attisdropped = src->attisdropped;
 	dst->attgenerated = (src->attgenerated != '\0');
-	dst->attnullability = !src->attnotnull ? ATTNULLABLE_UNRESTRICTED :
-		ATTNULLABLE_UNKNOWN;
+	if (IsCatalogRelationOid(src->attrelid))
+		dst->attnullability = !src->attnotnull ? ATTNULLABLE_UNRESTRICTED :
+			ATTNULLABLE_VALID;
+	else
+		dst->attnullability = !src->attnotnull ? ATTNULLABLE_UNRESTRICTED :
+			ATTNULLABLE_UNKNOWN;
 
 	switch (src->attalign)
 	{
diff --git a/src/backend/utils/cache/relcache.c b/src/backend/utils/cache/relcache.c
index 4128cfa8314..483f19711f0 100644
--- a/src/backend/utils/cache/relcache.c
+++ b/src/backend/utils/cache/relcache.c
@@ -704,7 +704,7 @@ RelationBuildTupleDesc(Relation relation)
 		 * CompactAttribute array.  In catalog relations however, any not-null
 		 * constraint is necessarily valid.
 		 */
-		for (int i = 0; i < relation->rd_rel->relnatts - 1; i++)
+		for (int i = 0; i < relation->rd_rel->relnatts; i++)
 		{
 			CompactAttribute *attr;
 
-- 
2.43.0

